@@ -20,8 +20,12 @@
 #include <QQmlEngine>
 #include <QTimer>
 #if HAVE_X11
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
 #include <QX11Info>
 #endif
+#endif
 
 #include <KActivities/ResourceInstance>
 #include <KFileItem>
@@ -35,7 +39,6 @@
 #include <KStartupInfo>
 
 #include <KActivities/Stats/Cleaning>
-#include <KActivities/Stats/ResultModel>
 #include <KActivities/Stats/Terms>
 #include <KWindowSystem>
 
@@ -188,7 +191,8 @@ QVariant RecentUsageModel::data(const QModelIndex &index, int role) const
     if (resource.startsWith(QLatin1String("applications:"))) {
         return appData(resource, role);
     } else {
-        return docData(resource, role);
+        const QString &mimeType = rowValueAt(index.row(), ResultModel::MimeType).toString();
+        return docData(resource, role, mimeType);
     }
 }
 
@@ -258,7 +262,7 @@ QModelIndex RecentUsageModel::findPlaceForKFileItem(const KFileItem &fileItem) c
     return QModelIndex();
 }
 
-QVariant RecentUsageModel::docData(const QString &resource, int role) const
+QVariant RecentUsageModel::docData(const QString &resource, int role, const QString &mimeType) const
 {
     QUrl url(resource);
 
@@ -268,7 +272,11 @@ QVariant RecentUsageModel::docData(const QString &resource, int role) const
 
     auto getFileItem = [=]() {
         // Avoid calling QT_LSTAT and accessing recent documents
-        return KFileItem(url, KFileItem::SkipMimeTypeFromContent);
+        if (mimeType.simplified().isEmpty()) {
+            return KFileItem(url, KFileItem::SkipMimeTypeFromContent);
+        } else {
+            return KFileItem(url, mimeType);
+        }
     };
 
     if (!url.isValid()) {
@@ -305,12 +313,12 @@ QVariant RecentUsageModel::docData(const QString &resource, int role) const
                 // if the current item is a place
                 return QString();
             }
-            desc.truncate(desc.lastIndexOf(QChar('/')));
+            desc.truncate(desc.lastIndexOf(QLatin1Char('/')));
             const auto text = m_placesModel->text(index);
             desc.replace(0, parentUrl.path().length(), text);
         } else {
             // remove filename
-            desc.truncate(desc.lastIndexOf(QChar('/')));
+            desc.truncate(desc.lastIndexOf(QLatin1Char('/')));
         }
         return desc;
     } else if (role == Kicker::UrlRole) {
@@ -350,7 +358,7 @@ bool RecentUsageModel::trigger(int row, const QString &actionId, const QVariant
         const QString &mimeType = rowValueAt(row, ResultModel::MimeType).toString();
 
         if (!resource.startsWith(QLatin1String("applications:"))) {
-            const QUrl resourceUrl = docData(resource, Kicker::UrlRole).toUrl();
+            const QUrl resourceUrl = docData(resource, Kicker::UrlRole, mimeType).toUrl();
 
             auto job = new KIO::OpenUrlJob(resourceUrl);
             job->setRunExecutables(false);
